@@ -169,13 +169,13 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 				if (fields->size == 0) {
 					fields->size = 16;
 					fields->used = 0;
-					fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+					fields->ptr = malloc(fields->size * sizeof(format_field * ));
 				} else if (fields->used == fields->size) {
 					fields->size += 16;
-					fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+					fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 				}
 
-				fields->ptr[fields->used] = malloc(sizeof(format_fields));
+				fields->ptr[fields->used] = malloc(sizeof(format_field));
 				fields->ptr[fields->used]->type = FIELD_STRING;
 				fields->ptr[fields->used]->string = buffer_init();
 
@@ -189,10 +189,10 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 			if (fields->size == 0) {
 				fields->size = 16;
 				fields->used = 0;
-				fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+				fields->ptr = malloc(fields->size * sizeof(format_field * ));
 			} else if (fields->used == fields->size) {
 				fields->size += 16;
-				fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+				fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 			}
 
 			/* search for the terminating command */
@@ -211,7 +211,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = NULL;
@@ -258,7 +258,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = buffer_init();
@@ -291,7 +291,7 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 
 					/* found key */
 
-					fields->ptr[fields->used] = malloc(sizeof(format_fields));
+					fields->ptr[fields->used] = malloc(sizeof(format_field));
 					fields->ptr[fields->used]->type = FIELD_FORMAT;
 					fields->ptr[fields->used]->field = fmap[j].type;
 					fields->ptr[fields->used]->string = NULL;
@@ -321,13 +321,13 @@ int accesslog_parse_format(server *srv, format_fields *fields, buffer *format) {
 		if (fields->size == 0) {
 			fields->size = 16;
 			fields->used = 0;
-			fields->ptr = malloc(fields->size * sizeof(format_fields * ));
+			fields->ptr = malloc(fields->size * sizeof(format_field * ));
 		} else if (fields->used == fields->size) {
 			fields->size += 16;
-			fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_fields * ));
+			fields->ptr = realloc(fields->ptr, fields->size * sizeof(format_field * ));
 		}
 
-		fields->ptr[fields->used] = malloc(sizeof(format_fields));
+		fields->ptr[fields->used] = malloc(sizeof(format_field));
 		fields->ptr[fields->used]->type = FIELD_STRING;
 		fields->ptr[fields->used]->string = buffer_init();
 
@@ -356,7 +356,7 @@ FREE_FUNC(mod_accesslog_free) {
 				if (s->use_syslog) {
 # ifdef HAVE_SYSLOG_H
 					if (s->access_logbuffer->used > 2) {
-						syslog(LOG_INFO, "%*s", s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
+						syslog(LOG_INFO, "%*s", (int) s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
 					}
 # endif
 				} else if (s->log_access_fd != -1) {
@@ -433,7 +433,7 @@ SETDEFAULTS_FUNC(log_access_open) {
 		if (i == 0 && buffer_is_empty(s->format)) {
 			/* set a default logfile string */
 
-			buffer_copy_string(s->format, "%h %V %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\"");
+			buffer_copy_string_len(s->format, CONST_STR_LEN("%h %V %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\""));
 		}
 
 		/* parse */
@@ -473,7 +473,7 @@ SETDEFAULTS_FUNC(log_access_open) {
 			continue;
 		}
 
-		if (buffer_is_empty(s->access_logfile)) continue;
+		if (s->access_logfile->used < 2) continue;
 
 		if (s->access_logfile->ptr[0] == '|') {
 #ifdef HAVE_FORK
@@ -498,6 +498,8 @@ SETDEFAULTS_FUNC(log_access_open) {
 				/* not needed */
 				close(to_log_fds[1]);
 
+				openDevNull(STDERR_FILENO);
+
 				/* we don't need the client socket */
 				for (i = 3; i < 256; i++) {
 					close(i);
@@ -538,8 +540,9 @@ SETDEFAULTS_FUNC(log_access_open) {
 
 			return HANDLER_ERROR;
 		}
+#ifdef FD_CLOEXEC
 		fcntl(s->log_access_fd, F_SETFD, FD_CLOEXEC);
-
+#endif
 	}
 
 	return HANDLER_GO_ON;
@@ -559,7 +562,7 @@ SIGHUP_FUNC(log_access_cycle) {
 #ifdef HAVE_SYSLOG_H
 				if (s->access_logbuffer->used > 2) {
 					/* syslog appends a \n on its own */
-					syslog(LOG_INFO, "%*s", s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
+					syslog(LOG_INFO, "%*s", (int) s->access_logbuffer->used - 2, s->access_logbuffer->ptr);
 				}
 #endif
 			} else if (s->log_access_fd != -1) {
@@ -570,7 +573,7 @@ SIGHUP_FUNC(log_access_cycle) {
 		}
 
 		if (s->use_syslog == 0 &&
-		    !buffer_is_empty(s->access_logfile) &&
+		    s->access_logfile->used > 1 &&
 		    s->access_logfile->ptr[0] != '|') {
 
 			close(s->log_access_fd);
@@ -582,6 +585,9 @@ SIGHUP_FUNC(log_access_cycle) {
 
 				return HANDLER_ERROR;
 			}
+#ifdef FD_CLOEXEC
+			fcntl(s->log_access_fd, F_SETFD, FD_CLOEXEC);
+#endif
 		}
 	}
 
@@ -647,9 +653,12 @@ REQUESTDONE_FUNC(log_access_write) {
 
 	mod_accesslog_patch_connection(srv, con, p);
 
+	/* No output device, nothing to do */
+	if (!p->conf.use_syslog && p->conf.log_access_fd == -1) return HANDLER_GO_ON;
+
 	b = p->conf.access_logbuffer;
 	if (b->used == 0) {
-		buffer_copy_string(b, "");
+		buffer_copy_string_len(b, CONST_STR_LEN(""));
 	}
 
 	for (j = 0; j < p->conf.parsed_format->used; j++) {
@@ -674,23 +683,23 @@ REQUESTDONE_FUNC(log_access_write) {
 					localtime_r(&(srv->cur_ts), &tm);
 					strftime(p->conf.ts_accesslog_str->ptr, p->conf.ts_accesslog_str->size - 1, "[%d/%b/%Y:%H:%M:%S ", &tm);
 # else
-					strftime(p->conf.ts_accesslog_str->ptr, p->conf.ts_accesslog_str->size - 1, "[%d/%b/%Y:%H:%M:%S ", localtime_r(&(srv->cur_ts)));
+					strftime(p->conf.ts_accesslog_str->ptr, p->conf.ts_accesslog_str->size - 1, "[%d/%b/%Y:%H:%M:%S ", localtime(&(srv->cur_ts)));
 # endif
 					p->conf.ts_accesslog_str->used = strlen(p->conf.ts_accesslog_str->ptr) + 1;
 
-					buffer_append_string(p->conf.ts_accesslog_str, tm.tm_gmtoff >= 0 ? "+" : "-");
+					buffer_append_string_len(p->conf.ts_accesslog_str, tm.tm_gmtoff >= 0 ? "+" : "-", 1);
 
 					scd = abs(tm.tm_gmtoff);
 					hrs = scd / 3600;
 					min = (scd % 3600) / 60;
 
 					/* hours */
-					if (hrs < 10) buffer_append_string(p->conf.ts_accesslog_str, "0");
+					if (hrs < 10) buffer_append_string_len(p->conf.ts_accesslog_str, CONST_STR_LEN("0"));
 					buffer_append_long(p->conf.ts_accesslog_str, hrs);
 
-					if (min < 10) buffer_append_string(p->conf.ts_accesslog_str, "0");
+					if (min < 10) buffer_append_string_len(p->conf.ts_accesslog_str, CONST_STR_LEN("0"));
 					buffer_append_long(p->conf.ts_accesslog_str, min);
-					BUFFER_APPEND_STRING_CONST(p->conf.ts_accesslog_str, "]");
+					buffer_append_string_len(p->conf.ts_accesslog_str, CONST_STR_LEN("]"));
 #else
 #ifdef HAVE_GMTIME_R
 					gmtime_r(&(srv->cur_ts), &tm);
@@ -717,13 +726,13 @@ REQUESTDONE_FUNC(log_access_write) {
 				break;
 			case FORMAT_REMOTE_IDENT:
 				/* ident */
-				BUFFER_APPEND_STRING_CONST(b, "-");
+				buffer_append_string_len(b, CONST_STR_LEN("-"));
 				break;
 			case FORMAT_REMOTE_USER:
 				if (con->authed_user->used > 1) {
 					buffer_append_string_buffer(b, con->authed_user);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_REQUEST_LINE:
@@ -740,42 +749,42 @@ REQUESTDONE_FUNC(log_access_write) {
 					buffer_append_off_t(b,
 							    con->bytes_written - con->bytes_header <= 0 ? 0 : con->bytes_written - con->bytes_header);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_HEADER:
 				if (NULL != (ds = (data_string *)array_get_element(con->request.headers, p->conf.parsed_format->ptr[j]->string->ptr))) {
 					buffer_append_string_buffer(b, ds->value);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_RESPONSE_HEADER:
 				if (NULL != (ds = (data_string *)array_get_element(con->response.headers, p->conf.parsed_format->ptr[j]->string->ptr))) {
 					buffer_append_string_buffer(b, ds->value);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_FILENAME:
 				if (con->physical.path->used > 1) {
 					buffer_append_string_buffer(b, con->physical.path);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_BYTES_OUT:
 				if (con->bytes_written > 0) {
 					buffer_append_off_t(b, con->bytes_written);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_BYTES_IN:
 				if (con->bytes_read > 0) {
 					buffer_append_off_t(b, con->bytes_read);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_TIME_USED:
@@ -785,28 +794,35 @@ REQUESTDONE_FUNC(log_access_write) {
 				if (con->server_name->used > 1) {
 					buffer_append_string_buffer(b, con->server_name);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_HTTP_HOST:
 				if (con->uri.authority->used > 1) {
 					buffer_append_string_buffer(b, con->uri.authority);
 				} else {
-					BUFFER_APPEND_STRING_CONST(b, "-");
+					buffer_append_string_len(b, CONST_STR_LEN("-"));
 				}
 				break;
 			case FORMAT_REQUEST_PROTOCOL:
-				buffer_append_string(b,
-						     con->request.http_version == HTTP_VERSION_1_1 ? "HTTP/1.1" : "HTTP/1.0");
+				buffer_append_string_len(b,
+						     con->request.http_version == HTTP_VERSION_1_1 ? "HTTP/1.1" : "HTTP/1.0", 8);
 				break;
 			case FORMAT_REQUEST_METHOD:
 				buffer_append_string(b, get_http_method_name(con->request.http_method));
 				break;
 			case FORMAT_PERCENT:
-				buffer_append_string(b, "%");
+				buffer_append_string_len(b, CONST_STR_LEN("%"));
 				break;
 			case FORMAT_SERVER_PORT:
-				buffer_append_long(b, srv->srvconf.port);
+				{
+					char *colon = strchr(((server_socket*)(con->srv_socket))->srv_token->ptr, ':');
+					if (colon) {
+						buffer_append_string(b, colon+1);
+					} else {
+						buffer_append_long(b, srv->srvconf.port);
+					}
+				}
 				break;
 			case FORMAT_QUERY_STRING:
 				buffer_append_string_buffer(b, con->uri.query);
@@ -816,8 +832,8 @@ REQUESTDONE_FUNC(log_access_write) {
 				break;
 			case FORMAT_CONNECTION_STATUS:
 				switch(con->keep_alive) {
-				case 0: buffer_append_string(b, "-"); break;
-				default: buffer_append_string(b, "+"); break;
+				case 0: buffer_append_string_len(b, CONST_STR_LEN("-")); break;
+				default: buffer_append_string_len(b, CONST_STR_LEN("+")); break;
 				}
 				break;
 			default:
@@ -837,7 +853,7 @@ REQUESTDONE_FUNC(log_access_write) {
 		}
 	}
 
-	BUFFER_APPEND_STRING_CONST(b, "\n");
+	buffer_append_string_len(b, CONST_STR_LEN("\n"));
 
 	if (p->conf.use_syslog ||  /* syslog doesn't cache */
 	    (p->conf.access_logfile->used && p->conf.access_logfile->ptr[0] == '|') || /* pipes don't cache */
@@ -847,7 +863,7 @@ REQUESTDONE_FUNC(log_access_write) {
 #ifdef HAVE_SYSLOG_H
 			if (b->used > 2) {
 				/* syslog appends a \n on its own */
-				syslog(LOG_INFO, "%*s", b->used - 2, b->ptr);
+				syslog(LOG_INFO, "%*s", (int) b->used - 2, b->ptr);
 			}
 #endif
 		} else if (p->conf.log_access_fd != -1) {
