@@ -115,7 +115,7 @@ static int mod_auth_patch_connection(server *srv, connection *con, mod_auth_plug
 	PATCH(auth_ldap_starttls);
 	PATCH(auth_ldap_allow_empty_pw);
 #ifdef USE_LDAP
-	PATCH(ldap);
+	p->anon_conf = s;
 	PATCH(ldap_filter_pre);
 	PATCH(ldap_filter_post);
 #endif
@@ -149,18 +149,24 @@ static int mod_auth_patch_connection(server *srv, connection *con, mod_auth_plug
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.hostname"))) {
 				PATCH(auth_ldap_hostname);
 #ifdef USE_LDAP
-				PATCH(ldap);
-				PATCH(ldap_filter_pre);
-				PATCH(ldap_filter_post);
+				p->anon_conf = s;
 #endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.base-dn"))) {
 				PATCH(auth_ldap_basedn);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.filter"))) {
 				PATCH(auth_ldap_filter);
+#ifdef USE_LDAP
+				PATCH(ldap_filter_pre);
+				PATCH(ldap_filter_post);
+#endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.ca-file"))) {
 				PATCH(auth_ldap_cafile);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.starttls"))) {
 				PATCH(auth_ldap_starttls);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.bind-dn"))) {
+				PATCH(auth_ldap_binddn);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.bind-pw"))) {
+				PATCH(auth_ldap_bindpw);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("auth.backend.ldap.allow-empty-pw"))) {
 				PATCH(auth_ldap_allow_empty_pw);
 			}
@@ -248,6 +254,7 @@ static handler_t mod_auth_uri_handler(server *srv, connection *con, void *p_d) {
 				if (0 == strcmp(method->value->ptr, "digest")) {
 					if (-1 == (auth_satisfied = http_auth_digest_check(srv, con, p, req, con->uri.path, auth_realm+1))) {
 						con->http_status = 400;
+						con->mode = DIRECT;
 
 						/* a field was missing */
 
@@ -268,22 +275,23 @@ static handler_t mod_auth_uri_handler(server *srv, connection *con, void *p_d) {
 		realm = (data_string *)array_get_element(req, "realm");
 
 		con->http_status = 401;
+		con->mode = DIRECT;
 
 		if (0 == strcmp(method->value->ptr, "basic")) {
-			buffer_copy_string(p->tmp_buf, "Basic realm=\"");
+			buffer_copy_string_len(p->tmp_buf, CONST_STR_LEN("Basic realm=\""));
 			buffer_append_string_buffer(p->tmp_buf, realm->value);
-			buffer_append_string(p->tmp_buf, "\"");
+			buffer_append_string_len(p->tmp_buf, CONST_STR_LEN("\""));
 
 			response_header_insert(srv, con, CONST_STR_LEN("WWW-Authenticate"), CONST_BUF_LEN(p->tmp_buf));
 		} else if (0 == strcmp(method->value->ptr, "digest")) {
 			char hh[33];
 			http_auth_digest_generate_nonce(srv, p, srv->tmp_buf, hh);
 
-			buffer_copy_string(p->tmp_buf, "Digest realm=\"");
+			buffer_copy_string_len(p->tmp_buf, CONST_STR_LEN("Digest realm=\""));
 			buffer_append_string_buffer(p->tmp_buf, realm->value);
-			buffer_append_string(p->tmp_buf, "\", nonce=\"");
+			buffer_append_string_len(p->tmp_buf, CONST_STR_LEN("\", nonce=\""));
 			buffer_append_string(p->tmp_buf, hh);
-			buffer_append_string(p->tmp_buf, "\", qop=\"auth\"");
+			buffer_append_string_len(p->tmp_buf, CONST_STR_LEN("\", qop=\"auth\""));
 
 			response_header_insert(srv, con, CONST_STR_LEN("WWW-Authenticate"), CONST_BUF_LEN(p->tmp_buf));
 		} else {
@@ -393,6 +401,23 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 			}
 		}
 
+#ifdef USE_LDAP
+		if (s->auth_ldap_filter->used) {
+			char *dollar;
+
+			/* parse filter */
+
+			if (NULL == (dollar = strchr(s->auth_ldap_filter->ptr, '$'))) {
+				log_error_write(srv, __FILE__, __LINE__, "s", "ldap: auth.backend.ldap.filter is missing a replace-operator '$'");
+
+				return HANDLER_ERROR;
+			}
+
+			buffer_copy_string_len(s->ldap_filter_pre, s->auth_ldap_filter->ptr, dollar - s->auth_ldap_filter->ptr);
+			buffer_copy_string(s->ldap_filter_post, dollar+1);
+		}
+#endif
+
 		/* no auth.require for this section */
 		if (NULL == (da = (data_array *)array_get_element(ca, "auth.require"))) continue;
 
@@ -479,21 +504,21 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 
 				ds = data_string_init();
 
-				buffer_copy_string(ds->key, "method");
+				buffer_copy_string_len(ds->key, CONST_STR_LEN("method"));
 				buffer_copy_string(ds->value, method);
 
 				array_insert_unique(a->value, (data_unset *)ds);
 
 				ds = data_string_init();
 
-				buffer_copy_string(ds->key, "realm");
+				buffer_copy_string_len(ds->key, CONST_STR_LEN("realm"));
 				buffer_copy_string(ds->value, realm);
 
 				array_insert_unique(a->value, (data_unset *)ds);
 
 				ds = data_string_init();
 
-				buffer_copy_string(ds->key, "require");
+				buffer_copy_string_len(ds->key, CONST_STR_LEN("require"));
 				buffer_copy_string(ds->value, require);
 
 				array_insert_unique(a->value, (data_unset *)ds);
@@ -502,102 +527,91 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 			}
 		}
 
-		switch(s->auth_backend) {
+		switch(s->auth_ldap_hostname->used) {
 		case AUTH_BACKEND_LDAP: {
 			handler_t ret = auth_ldap_init(srv, s);
 			if (ret == HANDLER_ERROR)
 				return (ret);
-                        break;
+			break;
 		}
-                default:
-                        break;
-                }
-        }
+		default:
+			break;
+		}
+	}
 
-        return HANDLER_GO_ON;
+	return HANDLER_GO_ON;
 }
 
 handler_t auth_ldap_init(server *srv, mod_auth_plugin_config *s) {
 #ifdef USE_LDAP
-			int ret;
+	int ret;
 #if 0
-			if (s->auth_ldap_basedn->used == 0) {
-				log_error_write(srv, __FILE__, __LINE__, "s", "ldap: auth.backend.ldap.base-dn has to be set");
+	if (s->auth_ldap_basedn->used == 0) {
+		log_error_write(srv, __FILE__, __LINE__, "s", "ldap: auth.backend.ldap.base-dn has to be set");
+
+		return HANDLER_ERROR;
+	}
+#endif
+
+	if (s->auth_ldap_hostname->used) {
+		/* free old context */
+		if (NULL != s->ldap) ldap_unbind_s(s->ldap);
+
+		if (NULL == (s->ldap = ldap_init(s->auth_ldap_hostname->ptr, LDAP_PORT))) {
+			log_error_write(srv, __FILE__, __LINE__, "ss", "ldap ...", strerror(errno));
+
+			return HANDLER_ERROR;
+		}
+
+		ret = LDAP_VERSION3;
+		if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(s->ldap, LDAP_OPT_PROTOCOL_VERSION, &ret))) {
+			log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
+
+			return HANDLER_ERROR;
+		}
+
+		if (s->auth_ldap_starttls) {
+			/* if no CA file is given, it is ok, as we will use encryption
+				* if the server requires a CAfile it will tell us */
+			if (!buffer_is_empty(s->auth_ldap_cafile)) {
+				if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(NULL, LDAP_OPT_X_TLS_CACERTFILE,
+								s->auth_ldap_cafile->ptr))) {
+					log_error_write(srv, __FILE__, __LINE__, "ss",
+							"Loading CA certificate failed:", ldap_err2string(ret));
+
+					return HANDLER_ERROR;
+				}
+			}
+
+			if (LDAP_OPT_SUCCESS != (ret = ldap_start_tls_s(s->ldap, NULL,  NULL))) {
+				log_error_write(srv, __FILE__, __LINE__, "ss", "ldap startTLS failed:", ldap_err2string(ret));
 
 				return HANDLER_ERROR;
 			}
-#endif
+		}
 
-			if (s->auth_ldap_filter->used) {
-				char *dollar;
 
-				/* parse filter */
+		/* 1. */
+		if (s->auth_ldap_binddn->used) {
+			if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(s->ldap, s->auth_ldap_binddn->ptr, s->auth_ldap_bindpw->ptr))) {
+				log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
 
-				if (NULL == (dollar = strchr(s->auth_ldap_filter->ptr, '$'))) {
-					log_error_write(srv, __FILE__, __LINE__, "s", "ldap: auth.backend.ldap.filter is missing a replace-operator '$'");
-
-					return HANDLER_ERROR;
-				}
-
-				buffer_copy_string_len(s->ldap_filter_pre, s->auth_ldap_filter->ptr, dollar - s->auth_ldap_filter->ptr);
-				buffer_copy_string(s->ldap_filter_post, dollar+1);
+				return HANDLER_ERROR;
 			}
+		} else {
+			if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(s->ldap, NULL, NULL))) {
+				log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
 
-			if (s->auth_ldap_hostname->used) {
-				if (NULL == (s->ldap = ldap_init(s->auth_ldap_hostname->ptr, LDAP_PORT))) {
-					log_error_write(srv, __FILE__, __LINE__, "ss", "ldap ...", strerror(errno));
-
-					return HANDLER_ERROR;
-				}
-
-				ret = LDAP_VERSION3;
-				if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(s->ldap, LDAP_OPT_PROTOCOL_VERSION, &ret))) {
-					log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
-
-					return HANDLER_ERROR;
-				}
-
-				if (s->auth_ldap_starttls) {
-					/* if no CA file is given, it is ok, as we will use encryption
-					 * if the server requires a CAfile it will tell us */
-					if (!buffer_is_empty(s->auth_ldap_cafile)) {
-						if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(NULL, LDAP_OPT_X_TLS_CACERTFILE,
-										s->auth_ldap_cafile->ptr))) {
-							log_error_write(srv, __FILE__, __LINE__, "ss",
-									"Loading CA certificate failed:", ldap_err2string(ret));
-
-							return HANDLER_ERROR;
-						}
-					}
-
-					if (LDAP_OPT_SUCCESS != (ret = ldap_start_tls_s(s->ldap, NULL,  NULL))) {
-						log_error_write(srv, __FILE__, __LINE__, "ss", "ldap startTLS failed:", ldap_err2string(ret));
-
-						return HANDLER_ERROR;
-					}
-				}
-
-
-				/* 1. */
-				if (s->auth_ldap_binddn->used) {
-					if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(s->ldap, s->auth_ldap_binddn->ptr, s->auth_ldap_bindpw->ptr))) {
-						log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
-
-						return HANDLER_ERROR;
-					}
-				} else {
-					if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(s->ldap, NULL, NULL))) {
-						log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
-
-						return HANDLER_ERROR;
-					}
-				}
+				return HANDLER_ERROR;
 			}
+		}
+	}
+	return HANDLER_GO_ON;
 #else
-			log_error_write(srv, __FILE__, __LINE__, "s", "no ldap support available");
-			return HANDLER_ERROR;
+	UNUSED(s);
+	log_error_write(srv, __FILE__, __LINE__, "s", "no ldap support available");
+	return HANDLER_ERROR;
 #endif
-		return HANDLER_GO_ON;
 }
 
 int mod_auth_plugin_init(plugin *p) {
