@@ -432,10 +432,10 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 
 	/* request line */
 	buffer_copy_string(b, get_http_method_name(con->request.http_method));
-	BUFFER_APPEND_STRING_CONST(b, " ");
+	buffer_append_string_len(b, CONST_STR_LEN(" "));
 
 	buffer_append_string_buffer(b, con->request.uri);
-	BUFFER_APPEND_STRING_CONST(b, " HTTP/1.0\r\n");
+	buffer_append_string_len(b, CONST_STR_LEN(" HTTP/1.0\r\n"));
 
 	proxy_append_header(con, "X-Forwarded-For", (char *)inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
 	/* http_host is NOT is just a pointer to a buffer
@@ -454,15 +454,16 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 
 		if (ds->value->used && ds->key->used) {
 			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Connection"))) continue;
+			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Proxy-Connection"))) continue;
 
 			buffer_append_string_buffer(b, ds->key);
-			BUFFER_APPEND_STRING_CONST(b, ": ");
+			buffer_append_string_len(b, CONST_STR_LEN(": "));
 			buffer_append_string_buffer(b, ds->value);
-			BUFFER_APPEND_STRING_CONST(b, "\r\n");
+			buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
 		}
 	}
 
-	BUFFER_APPEND_STRING_CONST(b, "\r\n");
+	buffer_append_string_len(b, CONST_STR_LEN("\r\n"));
 
 	hctx->wb->bytes_in += b->used - 1;
 	/* body */
@@ -652,7 +653,7 @@ static int proxy_demux_response(server *srv, handler_ctx *hctx) {
 			buffer_prepare_append(hctx->response, b + 1);
 			hctx->response->used = 1;
 		} else {
-			buffer_prepare_append(hctx->response, hctx->response->used + b);
+			buffer_prepare_append(hctx->response, b);
 		}
 
 		if (-1 == (r = read(hctx->fd, hctx->response->ptr + hctx->response->used - 1, b))) {
@@ -1060,6 +1061,8 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	data_array *extension = NULL;
 	size_t path_info_offset;
 
+	if (con->mode != DIRECT) return HANDLER_GO_ON;
+
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
 
@@ -1118,7 +1121,13 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		log_error_write(srv, __FILE__, __LINE__,  "s", "proxy - ext found");
 	}
 
-	switch(p->conf.balance) {
+	if (extension->value->used == 1) {
+		if ( ((data_proxy *)extension->value->data[0])->is_disabled ) {
+			ndx = -1;
+		} else {
+			ndx = 0;
+		}
+	} else if (extension->value->used != 0) switch(p->conf.balance) {
 	case PROXY_BALANCE_HASH:
 		/* hash balancing */
 
@@ -1175,7 +1184,9 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		}
 
 		break;
-	case PROXY_BALANCE_RR:
+	case PROXY_BALANCE_RR: {
+		data_proxy *host;
+
 		/* round robin */
 		if (p->conf.debug) {
 			log_error_write(srv, __FILE__, __LINE__,  "s",
@@ -1185,31 +1196,33 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		/* just to be sure */
 		assert(extension->value->used < INT_MAX);
 
-		for (k = 0, ndx = -1, max_usage = INT_MAX; k < extension->value->used; k++) {
-			data_proxy *host = (data_proxy *)extension->value->data[k];
+		host = (data_proxy *)extension->value->data[0];
 
-			if (host->is_disabled) continue;
+		/* Use last_used_ndx from first host in list */
+		k = host->last_used_ndx;
+		ndx = k + 1; /* use next host after the last one */
+		if (ndx < 0) ndx = 0;
 
-			/* first usable ndx */
-			if (max_usage == INT_MAX) {
-				max_usage = k;
+		/* Search first active host after last_used_ndx */
+		while ( ndx < (int) extension->value->used
+				&& (host = (data_proxy *)extension->value->data[ndx])->is_disabled ) ndx++;
+
+		if (ndx >= (int) extension->value->used) {
+			/* didn't found a higher id, wrap to the start */
+			for (ndx = 0; ndx < (int) k; ndx++) {
+				host = (data_proxy *)extension->value->data[ndx];
+				if (!host->is_disabled) break;
 			}
 
-			/* get next ndx */
-			if ((int)k > host->last_used_ndx) {
-				ndx = k;
-				host->last_used_ndx = k;
-
-				break;
-			}
+			/* No active host found */
+			if (host->is_disabled) ndx = -1;
 		}
 
-		/* didn't found a higher id, wrap to the start */
-		if (ndx == -1 && max_usage != INT_MAX) {
-			ndx = max_usage;
-		}
+		/* Save new index for next round */
+		((data_proxy *)extension->value->data[0])->last_used_ndx = ndx;
 
 		break;
+	}
 	default:
 		break;
 	}
