@@ -202,6 +202,15 @@ typedef struct {
 	 *
 	 */
 
+	/*
+	 * workaround for program when prefix="/"
+	 *
+	 * rule to build PATH_INFO is hardcoded for when check_local is disabled
+	 * enable this option to use the workaround
+	 *
+	 */
+
+	unsigned short fix_root_path_name;
 	ssize_t load; /* replace by host->load */
 
 	size_t max_id; /* corresponds most of the time to
@@ -233,6 +242,7 @@ typedef struct {
 typedef struct {
 	buffer *key; /* like .php */
 
+	int note_is_sent;
 	scgi_extension_host **hosts;
 
 	size_t used;
@@ -583,6 +593,7 @@ FREE_FUNC(mod_scgi_free) {
 
 static int env_add(char_array *env, const char *key, size_t key_len, const char *val, size_t val_len) {
 	char *dst;
+	size_t i;
 
 	if (!key || !val) return -1;
 
@@ -592,6 +603,15 @@ static int env_add(char_array *env, const char *key, size_t key_len, const char
 	/* add the \0 from the value */
 	memcpy(dst + key_len + 1, val, val_len + 1);
 
+	for (i = 0; i < env->used; i++) {
+		if (0 == strncmp(dst, env->ptr[i], key_len + 1)) {
+			/* don't care about free as we are in a forked child which is going to exec(...) */
+			/* free(env->ptr[i]); */
+			env->ptr[i] = dst;
+			return 0;
+		}
+	}
+
 	if (env->size == 0) {
 		env->size = 16;
 		env->ptr = malloc(env->size * sizeof(*env->ptr));
@@ -749,11 +769,20 @@ static int scgi_spawn_connection(server *srv,
 			env.size = 0;
 			env.used = 0;
 
+			if (scgi_fd != 0) {
+				close(0);
+				dup2(scgi_fd, 0);
+				close(scgi_fd);
+				scgi_fd = 0;
+			}
+
 			/* we don't need the client socket */
 			for (fd = 3; fd < 256; fd++) {
-				if (fd != 2 && fd != scgi_fd) close(fd);
+				close(fd);
 			}
 
+			openDevNull(STDERR_FILENO);
+
 			/* build clean environment */
 			if (host->bin_env_copy->used) {
 				for (i = 0; i < host->bin_env_copy->used; i++) {
@@ -794,7 +823,7 @@ static int scgi_spawn_connection(server *srv,
 			env.ptr[env.used] = NULL;
 
 			b = buffer_init();
-			buffer_copy_string(b, "exec ");
+			buffer_copy_string_len(b, CONST_STR_LEN("exec "));
 			buffer_append_string_buffer(b, host->bin_path);
 
 			/* exec the cgi */
@@ -968,6 +997,7 @@ SETDEFAULTS_FUNC(mod_scgi_set_defaults) {
 
 						{ "bin-environment",   NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },        /* 11 */
 						{ "bin-copy-environment", NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },     /* 12 */
+						{ "fix-root-scriptname",  NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },   /* 13 */
 
 
 						{ NULL,                NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
@@ -990,6 +1020,7 @@ SETDEFAULTS_FUNC(mod_scgi_set_defaults) {
 					df->max_load_per_proc = 1;
 					df->idle_timeout = 60;
 					df->disable_time = 60;
+					df->fix_root_path_name = 0;
 
 					fcv[0].destination = df->host;
 					fcv[1].destination = df->docroot;
@@ -1006,6 +1037,7 @@ SETDEFAULTS_FUNC(mod_scgi_set_defaults) {
 
 					fcv[11].destination = df->bin_env;
 					fcv[12].destination = df->bin_env_copy;
+					fcv[13].destination = &(df->fix_root_path_name);
 
 
 					if (0 != config_insert_values_internal(srv, da_host->value, fcv)) {
@@ -1084,7 +1116,7 @@ SETDEFAULTS_FUNC(mod_scgi_set_defaults) {
 								proc->port = df->port + pno;
 							} else {
 								buffer_copy_string_buffer(proc->socket, df->unixsocket);
-								buffer_append_string(proc->socket, "-");
+								buffer_append_string_len(proc->socket, CONST_STR_LEN("-"));
 								buffer_append_long(proc->socket, pno);
 							}
 
@@ -1155,11 +1187,6 @@ void scgi_connection_cleanup(server *srv, handler_ctx *hctx) {
 	p    = hctx->plugin_data;
 	con  = hctx->remote_conn;
 
-	if (con->mode != p->id) {
-		WP();
-		return;
-	}
-
 	if (hctx->fd != -1) {
 		fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
 		fdevent_unregister(srv->ev, hctx->fd);
@@ -1366,7 +1393,7 @@ static int scgi_env_add_request_headers(server *srv, connection *con, plugin_dat
 			buffer_reset(srv->tmp_buf);
 
 			if (0 != strcasecmp(ds->key->ptr, "CONTENT-TYPE")) {
-				BUFFER_COPY_STRING_CONST(srv->tmp_buf, "HTTP_");
+				buffer_copy_string_len(srv->tmp_buf, CONST_STR_LEN("HTTP_"));
 				srv->tmp_buf->used--;
 			}
 
@@ -1394,7 +1421,7 @@ static int scgi_env_add_request_headers(server *srv, connection *con, plugin_dat
 			buffer_prepare_append(srv->tmp_buf, ds->key->used + 2);
 			for (j = 0; j < ds->key->used - 1; j++) {
 				srv->tmp_buf->ptr[srv->tmp_buf->used++] =
-					isalpha((unsigned char)ds->key->ptr[j]) ?
+					light_isalnum((unsigned char)ds->key->ptr[j]) ?
 					toupper((unsigned char)ds->key->ptr[j]) : '_';
 			}
 			srv->tmp_buf->ptr[srv->tmp_buf->used++] = '\0';
@@ -1673,9 +1700,8 @@ static int scgi_response_parse(server *srv, connection *con, plugin_data *p, buf
 
 				status = strtol(s+9, NULL, 10);
 
-				if (con->http_status >= 100 &&
-				    con->http_status < 1000) {
-					/* we expected 3 digits and didn't got them */
+				if (status >= 100 && status < 1000) {
+					/* we expected 3 digits got them */
 					con->parsed_response |= HTTP_STATUS;
 					con->http_status = status;
 				}
@@ -1788,6 +1814,7 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 			int header_end = 0;
 			int cp, eol = EOL_UNSET;
 			size_t used = 0;
+			size_t hlen = 0;
 
 			buffer_append_string_buffer(hctx->response_header, hctx->response);
 
@@ -1810,6 +1837,7 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 
 					if (*(c+1) == '\n') {
 						header_end = 1;
+						hlen = cp + 2;
 						break;
 					}
 
@@ -1828,6 +1856,7 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 					    *(c+2) == '\r' &&
 					    *(c+3) == '\n') {
 						header_end = 1;
+						hlen = cp + 4;
 						break;
 					}
 
@@ -1849,12 +1878,11 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 					http_chunk_append_mem(srv, con, hctx->response_header->ptr, hctx->response_header->used);
 					joblist_append(srv, con);
 				} else {
-					size_t hlen = c - hctx->response_header->ptr + (eol == EOL_RN ? 4 : 2);
 					size_t blen = hctx->response_header->used - hlen - 1;
 
 					/* a small hack: terminate after at the second \r */
-					hctx->response_header->used = hlen + 1 - (eol == EOL_RN ? 2 : 1);
-					hctx->response_header->ptr[hlen - (eol == EOL_RN ? 2 : 1)] = '\0';
+					hctx->response_header->used = hlen;
+					hctx->response_header->ptr[hlen - 1] = '\0';
 
 					/* parse the response header */
 					scgi_response_parse(srv, con, p, hctx->response_header, eol);
@@ -1866,7 +1894,7 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 					}
 
 					if ((hctx->response->used != hlen) && blen > 0) {
-						http_chunk_append_mem(srv, con, c + (eol == EOL_RN ? 4: 2), blen + 1);
+						http_chunk_append_mem(srv, con, hctx->response_header->ptr + hlen, blen + 1);
 						joblist_append(srv, con);
 					}
 				}
@@ -2438,15 +2466,11 @@ static handler_t scgi_connection_close(server *srv, handler_ctx *hctx) {
 	p    = hctx->plugin_data;
 	con  = hctx->remote_conn;
 
-	if (con->mode != p->id) return HANDLER_GO_ON;
-
 	log_error_write(srv, __FILE__, __LINE__, "ssdsd",
 			"emergency exit: scgi:",
 			"connection-fd:", con->fd,
 			"fcgi-fd:", hctx->fd);
 
-
-
 	scgi_connection_cleanup(srv, hctx);
 
 	return HANDLER_FINISHED;
@@ -2548,13 +2572,13 @@ static handler_t scgi_handle_fdevent(void *s, void *ctx, int revents) {
 				con->mode = DIRECT;
 			} else {
 				/* response might have been already started, kill the connection */
-				scgi_connection_cleanup(srv, hctx);
-
 				log_error_write(srv, __FILE__, __LINE__, "ssdsd",
 						"response already sent out, termination connection",
 						"connection-fd:", con->fd,
 						"fcgi-fd:", hctx->fd);
 
+				scgi_connection_cleanup(srv, hctx);
+
 				connection_set_state(srv, con, CON_STATE_ERROR);
 			}
 
@@ -2668,10 +2692,12 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 	plugin_data *p = p_d;
 	size_t s_len;
 	int used = -1;
-	int ndx;
 	size_t k;
 	buffer *fn;
 	scgi_extension *extension = NULL;
+	scgi_extension_host *host = NULL;
+
+	if (con->mode != DIRECT) return HANDLER_GO_ON;
 
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
@@ -2712,95 +2738,59 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 	}
 
 	/* get best server */
-	for (k = 0, ndx = -1; k < extension->used; k++) {
-		scgi_extension_host *host = extension->hosts[k];
+	for (k = 0; k < extension->used; k++) {
+		scgi_extension_host *h = extension->hosts[k];
 
-		/* we should have at least one proc that can do somthing */
-		if (host->active_procs == 0) continue;
+		/* we should have at least one proc that can do something */
+		if (h->active_procs == 0) {
+			continue;
+		}
 
-		if (used == -1 || host->load < used) {
-			used = host->load;
+		if (used == -1 || h->load < used) {
+			used = h->load;
 
-			ndx = k;
+			host = h;
 		}
 	}
 
-	/* found a server */
-	if (ndx != -1) {
-		scgi_extension_host *host = extension->hosts[ndx];
+	if (!host) {
+		/* sorry, we don't have a server alive for this ext */
+		buffer_reset(con->physical.path);
+		con->http_status = 500;
 
-		/*
-		 * if check-local is disabled, use the uri.path handler
-		 *
-		 */
+		/* only send the 'no handler' once */
+		if (!extension->note_is_sent) {
+			extension->note_is_sent = 1;
 
-		/* init handler-context */
-		if (uri_path_handler) {
-			if (host->check_local == 0) {
-				handler_ctx *hctx;
-				char *pathinfo;
+			log_error_write(srv, __FILE__, __LINE__, "sbsbs",
+					"all handlers for ", con->uri.path,
+					"on", extension->key,
+					"are down.");
+		}
 
-				hctx = handler_ctx_init();
+		return HANDLER_FINISHED;
+	}
 
-				hctx->remote_conn      = con;
-				hctx->plugin_data      = p;
-				hctx->host             = host;
-				hctx->proc	       = NULL;
+	/* a note about no handler is not sent yet */
+	extension->note_is_sent = 0;
 
-				hctx->conf.exts        = p->conf.exts;
-				hctx->conf.debug       = p->conf.debug;
+	/*
+	 * if check-local is disabled, use the uri.path handler
+	 *
+	 */
 
-				con->plugin_ctx[p->id] = hctx;
-
-				host->load++;
-
-				con->mode = p->id;
-
-				if (con->conf.log_request_handling) {
-					log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_scgi");
-				}
-
-				/* the prefix is the SCRIPT_NAME,
-				 * everthing from start to the next slash
-				 * this is important for check-local = "disable"
-				 *
-				 * if prefix = /admin.fcgi
-				 *
-				 * /admin.fcgi/foo/bar
-				 *
-				 * SCRIPT_NAME = /admin.fcgi
-				 * PATH_INFO   = /foo/bar
-				 *
-				 * if prefix = /fcgi-bin/
-				 *
-				 * /fcgi-bin/foo/bar
-				 *
-				 * SCRIPT_NAME = /fcgi-bin/foo
-				 * PATH_INFO   = /bar
-				 *
-				 */
-
-				/* the rewrite is only done for /prefix/? matches */
-				if (extension->key->ptr[0] == '/' &&
-				    con->uri.path->used > extension->key->used &&
-				    NULL != (pathinfo = strchr(con->uri.path->ptr + extension->key->used - 1, '/'))) {
-					/* rewrite uri.path and pathinfo */
-
-					buffer_copy_string(con->request.pathinfo, pathinfo);
-
-					con->uri.path->used -= con->request.pathinfo->used - 1;
-					con->uri.path->ptr[con->uri.path->used - 1] = '\0';
-				}
-			}
-			return HANDLER_GO_ON;
-		} else {
+	/* init handler-context */
+	if (uri_path_handler) {
+		if (host->check_local == 0) {
 			handler_ctx *hctx;
+			char *pathinfo;
+
 			hctx = handler_ctx_init();
 
 			hctx->remote_conn      = con;
 			hctx->plugin_data      = p;
 			hctx->host             = host;
-			hctx->proc             = NULL;
+			hctx->proc	       = NULL;
 
 			hctx->conf.exts        = p->conf.exts;
 			hctx->conf.debug       = p->conf.debug;
@@ -2812,22 +2802,69 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 			con->mode = p->id;
 
 			if (con->conf.log_request_handling) {
-				log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_fastcgi");
+				log_error_write(srv, __FILE__, __LINE__, "s",
+				"handling it in mod_fastcgi");
 			}
 
-			return HANDLER_GO_ON;
+			/* the prefix is the SCRIPT_NAME,
+			 * everything from start to the next slash
+			 * this is important for check-local = "disable"
+			 *
+			 * if prefix = /admin.fcgi
+			 *
+			 * /admin.fcgi/foo/bar
+			 *
+			 * SCRIPT_NAME = /admin.fcgi
+			 * PATH_INFO   = /foo/bar
+			 *
+			 * if prefix = /fcgi-bin/
+			 *
+			 * /fcgi-bin/foo/bar
+			 *
+			 * SCRIPT_NAME = /fcgi-bin/foo
+			 * PATH_INFO   = /bar
+			 *
+			 */
+
+			/* the rewrite is only done for /prefix/? matches */
+			if (extension->key->ptr[0] == '/' &&
+			    con->uri.path->used > extension->key->used &&
+			    NULL != (pathinfo = strchr(con->uri.path->ptr + extension->key->used - 1, '/'))) {
+				/* rewrite uri.path and pathinfo */
+
+				buffer_copy_string(con->request.pathinfo, pathinfo);
+
+				con->uri.path->used -= con->request.pathinfo->used - 1;
+				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
+			} else if (host->fix_root_path_name && extension->key->ptr[0] == '/' && extension->key->ptr[1] == '\0') {
+				buffer_copy_string(con->request.pathinfo, con->uri.path->ptr);
+				con->uri.path->used = 1;
+				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
+			}
 		}
 	} else {
-		/* no handler found */
-		buffer_reset(con->physical.path);
-		con->http_status = 500;
+		handler_ctx *hctx;
+		hctx = handler_ctx_init();
 
-		log_error_write(srv, __FILE__, __LINE__,  "sb",
-				"no fcgi-handler found for:",
-				fn);
+		hctx->remote_conn      = con;
+		hctx->plugin_data      = p;
+		hctx->host             = host;
+		hctx->proc             = NULL;
 
-		return HANDLER_FINISHED;
+		hctx->conf.exts        = p->conf.exts;
+		hctx->conf.debug       = p->conf.debug;
+
+		con->plugin_ctx[p->id] = hctx;
+
+		host->load++;
+
+		con->mode = p->id;
+
+		if (con->conf.log_request_handling) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_fastcgi");
+		}
 	}
+
 	return HANDLER_GO_ON;
 }
 
@@ -2949,7 +2986,7 @@ TRIGGER_FUNC(mod_scgi_handle_trigger) {
 						fp->port = host->port + fp->id;
 					} else {
 						buffer_copy_string_buffer(fp->socket, host->unixsocket);
-						buffer_append_string(fp->socket, "-");
+						buffer_append_string_len(fp->socket, CONST_STR_LEN("-"));
 						buffer_append_long(fp->socket, fp->id);
 					}
 
