@@ -825,6 +825,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 
@@ -874,6 +875,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Expect")))) {
@@ -892,11 +894,12 @@ int http_request_parse(server *srv, connection *con) {
 								 *
 								 */
 
-								con->http_status = 417;
-								con->keep_alive = 0;
-
-								array_insert_unique(con->request.headers, (data_unset *)ds);
-								return 0;
+								if (srv->srvconf.reject_expect_100_with_417 && 0 == buffer_caseless_compare(CONST_BUF_LEN(ds->value), CONST_STR_LEN("100-continue"))) {
+									con->http_status = 417;
+									con->keep_alive = 0;
+									array_insert_unique(con->request.headers, (data_unset *)ds);
+									return 0;
+								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Host")))) {
 								if (!con->request.http_host) {
 									con->request.http_host = ds->value;
@@ -911,6 +914,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("If-Modified-Since")))) {
@@ -936,6 +940,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("If-None-Match")))) {
@@ -953,6 +958,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("Range")))) {
@@ -976,6 +982,7 @@ int http_request_parse(server *srv, connection *con) {
 												"request-header:\n",
 												con->request.request);
 									}
+									array_insert_unique(con->request.headers, (data_unset *)ds);
 									return 0;
 								}
 							}
@@ -1014,7 +1021,7 @@ int http_request_parse(server *srv, connection *con) {
 				/* strip leading WS */
 				if (value == cur) value = cur+1;
 			default:
-				if (*cur >= 0 && *cur < 32) {
+				if (*cur >= 0 && *cur < 32 && *cur != '\t') {
 					if (srv->srvconf.log_request_header_on_error) {
 						log_error_write(srv, __FILE__, __LINE__, "sds",
 								"invalid char in header", (int)*cur, "-> 400");
@@ -1129,8 +1136,8 @@ int http_request_parse(server *srv, connection *con) {
 			con->http_status = 413;
 			con->keep_alive = 0;
 
-			log_error_write(srv, __FILE__, __LINE__, "sds",
-					"request-size too long:", con->request.content_length, "-> 413");
+			log_error_write(srv, __FILE__, __LINE__, "sos",
+					"request-size too long:", (off_t) con->request.content_length, "-> 413");
 			return 0;
 		}
 
@@ -1144,8 +1151,8 @@ int http_request_parse(server *srv, connection *con) {
 			con->http_status = 413;
 			con->keep_alive = 0;
 
-			log_error_write(srv, __FILE__, __LINE__, "sds",
-					"request-size too long:", con->request.content_length, "-> 413");
+			log_error_write(srv, __FILE__, __LINE__, "sos",
+					"request-size too long:", (off_t) con->request.content_length, "-> 413");
 			return 0;
 		}
 
